// An OO interpretation of the solution to the Blizzard question
// in Assignment 05.  A clear demonstration of the elegance of
// grouping data together into classes.  Notice how many array
// reference operations we get rid of!

class Snowflake
{
  PShape shp;  // Which shape to draw
  float x;     // Current X location
  float y;     // Current Y location
  float rot;   // Current orientation
  float rvel;  // Current angular velocity
  float sc;    // Current scaling factor
  
  Snowflake()
  {
    // Constructor with no arguments -- make a random
    // flake.
    shp = flake_shapes[ int( random( 12 ) ) ];
    sc = lerp( min_scale, max_scale, 
      pow( random( 1.0 ), 10.0 ) );
    rot = random( TWO_PI );
    rvel = random( -0.1, 0.1 );
    x = random( width );
    y = random( -2.0*height, 0 );
  }
  
  void draw()
  {
    // Transform it into place.
    pushMatrix();
    translate( x, y );
    rotate( rot );
    scale( sc );
    
    // A cute little change -- if the flake is really
    // small, replace it with a circle.  This has the 
    // side effect of making the simulation much faster,
    // allowing us to get up to 1000 flakes.
    if( sc < 0.021 ) {
      ellipse( 0, 0, shp.width, shp.width );
    } else {
      translate( -shp.width/2.0, -shp.height/2.0 );
      shape( shp );
    }
    popMatrix();
  }
  
  void step()
  {
    // Update rotation
    rot += rvel;
    
    // Update position.  If we're past the bottom of the
    // screen move back to the top.
    
    // The amount you fall by is proportional to the size
    // of the flake.
    y += 0.1*shp.height*sc;
    
    if( y-0.5*shp.height*sc > height ) {
      y = -0.5*shp.height;
      // Also randomize the X location.  That should be
      // enough to make the blizzard look completely random.
      // In practice you can easily reset all the other random
      // parameters too.
      x = random( width );
    }
  }
}

// How many flakes will we be tracking?
int num_flakes;
PShape[] flake_shapes;
Snowflake[] flakes;

// Set limits for the size of a snowflake.
float min_scale = 0.02;
float max_scale = 0.25;

void setup()
{
  size( 500, 500 );
  flake_shapes = new PShape[ 12 ];
  for( int idx = 0; idx < 12; ++idx ) {
    flake_shapes[idx] = 
      loadShape( "snowflake" + (idx+1) + ".svg" );
    // Turn off automatic styling to allow for control
    // of a snowflake's transparency.
    flake_shapes[idx].disableStyle();
  }
  
  // Initialize a big pile of snowflakes
  num_flakes = 1000;
  flakes = new Snowflake[ num_flakes ];
  
  for( int idx = 0; idx < num_flakes; ++idx ) {
    flakes[idx] = new Snowflake();
  }
}

void draw()
{
  background( 200, 200, 255 );
  
  fill( 255, 180 );
  noStroke();
  
  // Draw each flake and then update its parameters.
  for( int idx = 0; idx < num_flakes; ++idx ) {
    flakes[idx].step();
    flakes[idx].draw();
  }
}

