// Draw a Sierpinski Gasket by explicitly tracking the corners of
// a triangle.  Then, to make the recursive calls, you can compute
// the corners of the three smaller triangles you want to recurse on
// and continue with them.  Overall the structure is similar to
// ContextGasket.  Some students may find that this one's easier
// to think about (no confusing geometric contexts); on the other
// hand, it requires a bit more typing.  Could be a useful model for
// the "Pinwheel tiling" question on Assignment 06.

// Do it using an explicit Point class.  Of course, we could also
// use the built-in PVector class.

float hr3 = sqrt(3.0)/2.0;
int levels = 5;

class Point
{
  float x;
  float y;
  
  Point( float ax, float ay )
  {
    x = ax;
    y = ay;
  }
  
  Point mid( Point other )
  {
    return new Point( 0.5*(x+other.x), 0.5*(y+other.y) );
  }
}

void setup()
{ 
  size( 500, int( 500 * hr3 ) );
}

void draw()
{
  background( 255 );
  noStroke();
  fill( 0 );

  // Because the window size was chosen to fit an equilateral
  // triangle, the starting triangle can be derived directly from
  // the window size.
  drawTriangle( levels, 
    new Point( width/2, 0 ),
    new Point( 0, height ),
    new Point( width, height ) );
}

void drawTriangle( int levs, Point a, Point b, Point c )
{
  if( levs == 0 ) {
    // Base case -- no more recursive levels to draw.
    // Just draw a triangle.
    
    triangle( a.x, a.y, b.x, b.y, c.x, c.y );
  } else {
    // Recursive case -- draw three recursive copies of
    // the gasket at the next level down.  Compute the
    // triangles bounding those recursive copies by computing
    // midpoints of the three points of the passed-in
    // triangle.
    
    Point mab = a.mid( b );
    Point mac = a.mid( c );
    Point mbc = b.mid( c );
    
    drawTriangle( levs - 1, a, mab, mac );
    drawTriangle( levs - 1, mab, b, mbc );
    drawTriangle( levs - 1, mac, mbc, c );
  }
}

void keyPressed()
{
  if( key == '-' ) {
    levels = max( 0, levels - 1 );
  } else if( key == '=' ) {
    ++levels;
  }
}

