// An Object-Oriented modification of the DirectManipMulti
// sample sketch from Module 03.

class Dot
{
  int x;
  int y;
  color col;
  
  // Constructor: make a new dot from its location and colour.
  Dot( int xx, int yy, color c )
  {
    x = xx;
    y = yy;
    col = c;
  }
  
  void draw()
  {
    fill( col );
    ellipse( x, y, 2*radius, 2*radius );
  }
  
  boolean hit( int mx, int my )
  {
    return dist( x, y, mx, my ) <= radius;
  }
  
  void move( int dx, int dy )
  {
    x += dx;
    y += dy;
  }
}

int num_dots = 15;
float radius = 20;

// Declare an array to hold all the dots that will appear
// on the screen.
Dot[] dots;

// Which of those dots are we dragging right now?  Don't use
// an index into the array -- instead, hold a reference to 
// the actual Dot itself.
Dot dragging;

void setup()
{
  size( 500, 500 );
  
  // Generate a bunch of random dots.
  
  // This line just creates the array.  It doesn't initialize
  // any actual dots.
  dots = new Dot[ num_dots ];

  for ( int idx = 0; idx < num_dots; ++idx ) {
    dots[idx] = new Dot(
      int(random( width )), int(random( height )),
      color( random(255), random(255), random(255) ) );
  }
  
  // When we're doing OO, we can use the special null value
  // to indicate that there's no active dot.
  dragging = null;
}

void draw()
{
  background( 220 );
  for ( int idx = 0; idx < num_dots; ++idx ) {
    // Compare the dragging variable with the current dot
    // to see if they're the same object.
    if( dots[idx] == dragging ) {
      stroke( #ff0000 );
    } else {
      stroke( #000000 );
    }
    dots[idx].draw();
  }
}

void mousePressed()
{
  // Need to do a hit test against all the dots.
  
  // At this stage in the course, let's remember to perform
  // hit testing in reverse order of drawing.
  for ( int idx = num_dots - 1; idx >= 0; --idx ) {
    if( dots[idx].hit( mouseX, mouseY ) ) {
      dragging = dots[idx];
      break;
    }
  }
}

void mouseDragged()
{
  if ( dragging != null ) {
    dragging.move( mouseX - pmouseX, mouseY - pmouseY );
  }
}

void mouseReleased()
{
  dragging = null;
}

