// An OO version of the Foam sketch from Module 06

class Circle
{
  float x;
  float y;
  float r;

  Circle( float xx, float yy, float rr )
  {
    x = xx;
    y = yy;
    r = rr;
  }

  boolean intersects( Circle other )
  {
    float d = dist( x, y, other.x, other.y );
    return (r+other.r) > d;
  }

  void draw()
  {
    float t = norm( r, 80, 0 );

    fill( lerp( 0, 75, sq( t ) ), 100, 100 );
    ellipse( x, y, 2*r, 2*r );
    return;
  }
}

// All the circles we know about up to this point
Circle[] circles;

// The current radius of circle we're trying to place.
float radius;

boolean maybeAddCircle( Circle circ )
{
  for ( int idx = 0; idx < circles.length; ++idx ) {
    if ( circles[idx].intersects( circ ) ) {
      return false;
    }
  }

  // Because we're using an array of our own custom class,
  // it turns out that the append() function doesn't work
  // the way we expect anymore, for reasons I don't want to
  // get into.  As a result, I need to take a few lines to
  // "manually" add the new circle to the array of circles. 
  // Dang.
  Circle[] ncircles = new Circle[ circles.length + 1 ];
  arrayCopy( circles, 0, ncircles, 0, circles.length );
  ncircles[ circles.length ] = circ;
  circles = ncircles;
  
  // This would work too, but it requires a "cast", which
  // I'd just as soon not talk about this term.
  // circles = (Circle[])append( circles, circ );
  
  return true;
}

void setup()
{
  size( 500, 500 );
  stroke( 0 );
  colorMode( HSB, 100 );
  initCircles();
}

void initCircles()
{
  circles = new Circle[0];
  radius = 80.0;
}

void draw()
{
  // Stop when the circles get small enough.
  if ( radius > 1 ) {
    // Try a thousand times to find a random circle location
    // that doesn't yet intersect other circles.  If you find 
    // it, draw it.  If you run out of tries, make the radius
    // slightly smaller.
    for ( int idx = 0; idx < 1000; ++idx ) {
      Circle ncirc = new Circle(
      random(width), random(height), radius );
      if ( maybeAddCircle( ncirc ) ) {
        ncirc.draw();
        return;
      }
    }

    radius = radius * 0.99;
  }
}

void keyPressed()
{
  initCircles();
  background( 0, 0, 80 );
}

